@@ -872,6 +872,11 @@ proc build_date_query_qualifier {tag d when} {
     if {$mode == "socket"} {
 	# XXX GNATS 4
         #set str "$gnatsd_commands($tag.$when) $date"
+	if { $when == "after" } {
+	    set str "$tag > \"$date\""
+	} {
+	    set str "$tag < \"$date\""
+	}
     } {
         set str "--$var=$date"
     }
@@ -1126,15 +1131,11 @@ proc save_query_cmd {mode} {
         textset Number $prid_list $Query(stextparent,Number)
         if {$mode != "save"} {
             set Query(default__query) [list proc default__query {flds} {return 1}]
-            if {$TkGnats(GNATS_ACCESS_METHOD) == "batch"} {
-		set Query(query_pr_opts) ""
-		foreach prid $prid_list {
-		    append Query(query_pr_opts) "Number==$prid_list|"
-		}
-		append Query(query_pr_opts) " "
-            } {
-                set Query(query_pr_opts) "$Query(query_mode) $prid_list"
-            }
+	    set Query(query_pr_opts) ""
+	    foreach prid $prid_list {
+		append Query(query_pr_opts) "Number==$prid_list|"
+	    }
+	    append Query(query_pr_opts) " "
             set Query(by_id_only) 1
             query_cmd $prid_list
             return 1
