@@ -1,6 +1,6 @@
 /* Common functions for each GNATS client.
+   Copyright (C) 1994, 1995, 1999, 2000, 2002 Free Software Foundation, Inc.
    Copyright (C) 2001 Dan Martinez
-   Copyright (C) 1994, 1995, 1999, 2000 Free Software Foundation, Inc.
    Contributed by Brendan Kehoe (brendan@cygnus.com).
 
 This file is part of GNU GNATS.
@@ -211,7 +211,7 @@ readPRListFromServer (void)
 	    {
 	      i = 1;
 	    }
-	  while (recvline[i] != '\0' && isspace ((int) recvline[i]))
+	  while (recvline[i] != '\0' && isspace ((int)(unsigned char) recvline[i]))
 	    {
 	      i++;
 	    }
@@ -778,9 +778,10 @@ scanEnv (char **user, char **passwd, char **hostname,
 	 int *port, char **database)
 {
   char *vals[5];
-  char *evar = getenv ("GNATSDB");
+  char *evar = *database == NULL ? getenv ("GNATSDB") : *database;
+  bool is_net_conn = databaseSpecIsNetConn (evar);
 
-  if (evar == NULL || evar[0] == '\0' || databaseSpecIsNetConn (evar))
+  if (evar == NULL || evar[0] == '\0' || is_net_conn)
     {
       if (*hostname == NULL)
 	{
@@ -908,9 +909,11 @@ scanEnv (char **user, char **passwd, char **hostname,
       *passwd = xstrdup ("*");
     }
 
-  if (*database == NULL)
+  if (*database == NULL && ! is_net_conn)
     {
-      *database = xstrdup ("default");
+      *database = ((evar == NULL || evar[0] == '\0')
+		   ? xstrdup ("default")
+		   : xstrdup (evar));
     }
 
   if (*port == -1)
@@ -1257,26 +1260,41 @@ netEditField (FILE *fp, const char *prnum, const char *fieldname,
       cmd = "REPL";
     }
   fprintf (serv_write, "%s %s %s\r\n", cmd, prnum, fieldname);
-  /* Have to send the field-change reason here. */
-  reply = get_reply (stdout);
-  if (reply == CODE_SEND_CHANGE_REASON)
-    {
-      if (reason)
-	fprintf (serv_write, "%s\r\n.\r\n", reason);
-      else
-	{
-	  fprintf (stderr, "%s: server wants a change-reason for %s,"
-		   " but none was supplied.\n", program_name, fieldname);
+
+  do {
+    reply = get_reply (stdout);
+    switch (reply)
+      {
+        case CODE_SEND_TEXT:
+          while ((line = read_line (fp, NULL)) != NULL)
+            {
+              write_multitext (serv_write, line, "\r\n");
+              free (line);
+            }
+          fprintf (serv_write, ".\r\n");
+	  break;
+
+        case CODE_SEND_CHANGE_REASON:
+          if (reason)
+	    fprintf (serv_write, "%s\r\n.\r\n", reason);
+          else
+	    {
+	      fprintf (stderr, "%s: server wants a change-reason for %s,"
+	         " but none was supplied.\n", program_name, fieldname);
+	      safe_exit ();
+	    }
+	  break;
+    
+	case CODE_OK:
+	  break;
+
+        default:
+	  fprintf (stderr, "%s: bad reply from server: %d.\n",
+		   program_name, reply);
 	  safe_exit ();
-	}
-    }
-  while ((line = read_line (fp, NULL)) != NULL)
-    {
-      write_multitext (serv_write, line, "\r\n");
-      free (line);
-    }
-  fprintf (serv_write, ".\r\n");
-  get_reply (stdout);
+	  break;
+      }
+  } while (reply != CODE_OK);
 }
 
 void
