@@ -29,12 +29,14 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 static int needs_analysis (PR *pr);
 
 static int run_atpr(PR *pr, AdmEntry *submitter, int submitter_rtime,
-		    struct tm *expired, char *bug_name, char *path,
-		    ErrorDesc *err);
+		    struct tm *expired, char *bug_name, ErrorDesc *err);
 
 static int getBugNumber        (const DatabaseInfo database, ErrorDesc *err);
 static char* derive_submitter  (PR *pr);
 
+static int missing_required_fields (const DatabaseInfo database, PR *pr,
+                                    ErrorDesc *err);
+
 
 /* Creates a PR file using the contents of PR.  If an error occurs, a
    negative value will be returned an an appropriate error code will
@@ -69,6 +71,11 @@ createNewPRFile (PR *pr, int flag_autocreate, ErrorDesc *err)
       return -1;
     }
 
+  if (missing_required_fields (database, pr, err) > 0)
+    {
+      return -1;
+    }
+
   /*  If we don't have a valid submitter ID, try to get it from the "From:"
       header.  If that doesn't work, use the default from the config file.  */
 
@@ -308,13 +315,37 @@ createNewPRFile (PR *pr, int flag_autocreate, ErrorDesc *err)
 	}
     }
 
-  /* XXX ??? !!! Do we want to do this always, or only if the state is
-     empty?  */
-  /* Make sure the PR is in the correct state. */
-  set_field (pr,
-	     STATE (database), 
-	     fieldDefForIndex(STATE (database))->default_value,
-	     err);
+  {
+    /* Ensure the PR has a valid STATE.  We set the default state
+       on any of these cases:
+       
+         1. field_value() returns NULL.
+	 
+	 2. if field_value returns the default value.  (field_value in
+	    4.0beta1 returns the default, even if it is not really set.)
+
+	 3. if set_field with the existing value fails.  If something
+	    used set_field() previously, it was already validated,
+	    but we call set_field to validate again to be sure.
+    */
+    char *val = (char *)field_value (pr, STATE (database));
+    char *default_val = fieldDefForIndex (STATE (database))->default_value;
+    
+    if (val != NULL)
+      {
+	val = xstrdup (val);
+      }
+    if (val == NULL
+	|| ! strcmp (val, default_val)
+	|| ! set_field (pr, STATE (database), val, err))
+      {
+	set_field (pr, STATE (database), default_val, err);
+      }
+    if (val != NULL)
+      {
+	free (val);
+      }
+  }
 
   /* Retrieve a unique bug number.  */
   bug_number = getBugNumber (database, err);
@@ -369,7 +400,7 @@ createNewPRFile (PR *pr, int flag_autocreate, ErrorDesc *err)
 	      && needs_analysis (pr))
 	    {
 	      if (run_atpr (pr, submitter, submitter_rtime, expired, bug_name, 
-			    prPath, err) != 0)
+			    err) != 0)
 		{
 		  bug_number = -1;
 		}
@@ -405,6 +436,28 @@ createNewPRFile (PR *pr, int flag_autocreate, ErrorDesc *err)
   return bug_number;
 }
 
+static int
+missing_required_fields (const DatabaseInfo database, PR *pr, ErrorDesc *err)
+{
+  int cnt = 0;
+  FieldList fields = getRequiredInputFields(database);
+
+  while (fields != NULL)
+    {
+      const char *fldval = get_field_value (pr, NULL, fields->ent, NULL, NULL);
+      if (value_is_empty (fldval))
+	{
+          setError (err, CODE_INVALID_PR_CONTENTS,
+                    "Required field %s missing from new PR - cannot submit.",
+                    complexFieldIndexToString (fields->ent),
+                    field_value (pr, NUMBER (pr->database)));
+	  cnt++;
+	}
+      fields = fields->next;
+    }
+  return cnt;
+}
+
 /* XXX ??? !!! FIXME Shouldn't be hardcoded.   */
 static int
 needs_analysis (PR *pr)
@@ -478,8 +531,7 @@ arg_quote (const char *s)
 
 static int
 run_atpr (PR *pr, AdmEntry *submitter, int submitter_rtime, 
-	  struct tm *expired, char *bug_name, char *path,
-	  ErrorDesc *err)
+	  struct tm *expired, char *bug_name, ErrorDesc *err)
 {
   char *at_pr;
   char *command = NULL;
@@ -496,7 +548,7 @@ run_atpr (PR *pr, AdmEntry *submitter, int submitter_rtime,
     {
       if (access (ats[i], X_OK) == 0)
 	{
-	  asprintf (&command, "%s %s", ats[i], buf);
+	  asprintf (&command, "%s %s  &> /dev/null", ats[i], buf);
 	  break;
 	}
     }
@@ -517,9 +569,9 @@ run_atpr (PR *pr, AdmEntry *submitter, int submitter_rtime,
 	  char *cont = arg_quote (submitter->admFields[SubmitterAdmContact]);
 	  char *pa = arg_quote (gnatsAdminMailAddr (pr->database));
       
-	  fprintf (p, "%s --database %s %d %s %s %s '%s' '%s' '%s'\n", at_pr,
+	  fprintf (p, "%s --database %s %d %s %s '%s' '%s' '%s'\n", at_pr,
 		   databaseName (pr->database),
-		   submitter_rtime, bug_name, path, 
+		   submitter_rtime, bug_name,
 		   submitter->admFields[SubmitterAdmKey],
 		   orig, cont, pa);
 
@@ -570,15 +622,16 @@ checkIfReply (PR *pr, ErrorDesc *err)
   regex.translate = case_fold;
   
   {
-    const char *const PAT = "\\<((PR[ \t/])|([-a-z0-9_+.]+)/)([0-9]+)";
+    const char *const PAT = "\\<((PR[ \t/])\\|([-a-z0-9_+.]+)/)([0-9]+)";
     re_compile_pattern (PAT, strlen (PAT), &regex);
   }
-  i = re_match (&regex, headerValue, strlen (headerValue), 0, &regs);
+  i = re_search (&regex, headerValue, strlen (headerValue), 0,
+		 strlen (headerValue), &regs);
   regex.translate = NULL;
   regfree (&regex);
   re_set_syntax (old_syntax);
 
-  if (i <= 0)
+  if (i < 0)
     {
       return NULL;
     }
