@@ -1,6 +1,6 @@
 ;;; gnats.el --- Emacs interface for GNATS.
 
-;; Copyright (C) 2000, 2001 Milan Zamazal
+;; Copyright (C) 2000, 2001, 2002 Milan Zamazal
 ;; Copyright (C) 2000 Bob Manson
 ;;
 ;; Contributed by Bob Manson (manson@juniper.net)
@@ -185,15 +185,9 @@ functions can modify it.")
   "Undo, but disabling motion-hooks and read-only attributes.
 Do the undo COUNT times."
   (interactive "P")
-  (let ((old-inhibit-read-only inhibit-read-only)
-	(old-motion-hooks inhibit-point-motion-hooks))
-    (unwind-protect
-	(progn
-	  (setq inhibit-point-motion-hooks t)
-	  (setq inhibit-read-only t)
-	  (advertised-undo count))
-      (setq inhibit-point-motion-hooks old-motion-hooks)
-      (setq inhibit-read-only old-inhibit-read-only))))
+  (let ((inhibit-read-only t)
+	(inhibit-point-motion-hooks t))
+    (advertised-undo count)))
 
 (defun gnats-mark-field-edited (field)
   "Mark FIELD as having been edited.
@@ -229,23 +223,17 @@ CURR-POINT must be a point within the field contents."
   "Replace FIELD's text in the edit buffer with NEW-TEXT."
   (let* ((curr-point (point))
 	 (start (gnats-find-field-start field curr-point))
-	 (end (next-single-property-change curr-point 'gnats-field-name)))
-    (let ((old-inhibit-read-only inhibit-read-only)
-	  (old-motion-hooks inhibit-point-motion-hooks))
-      (unwind-protect
-	  (progn
-	    (setq inhibit-read-only t)
-	    (setq inhibit-point-motion-hooks t)
-	    (delete-region start end)
-	    ;; Make the undo atomic.
-	    (if (and (listp buffer-undo-list) (null (car buffer-undo-list)))
-		(setq buffer-undo-list (cdr buffer-undo-list)))
-	    (goto-char start)
-	    (gnats-insert-text field new-text t
-			       (list 'read-only t 'mouse-face 'highlight
-				     'local-map gnats-edit-button-map)))
-	(setq inhibit-read-only old-inhibit-read-only)
-	(setq inhibit-point-motion-hooks old-motion-hooks)))))
+	 (end (next-single-property-change curr-point 'gnats-field-name))
+	 (inhibit-read-only t)
+	 (inhibit-point-motion-hooks t))
+    (delete-region start end)
+    ;; Make the undo atomic.
+    (if (and (listp buffer-undo-list) (null (car buffer-undo-list)))
+	(setq buffer-undo-list (cdr buffer-undo-list)))
+    (goto-char start)
+    (gnats-insert-text field new-text t
+		       (list 'read-only t 'mouse-face 'highlight
+			     'local-map gnats-edit-button-map))))
 
 (defun gnats-delete-field (field)
   "Remove FIELD entirely from the PR.
@@ -517,7 +505,7 @@ are applied, the PR contents are reloaded."
 	(setq field-list (cdr field-list))))
     (gnats-clear-edit-buffer)
     (gnats-get-pr (get gnats-server-conn 'pr-number))
-    (gnats-edit-mode)))
+    (gnats-edit-mode t)))
 
 (defun gnats-apply-or-submit ()
   "If in edit mode apply the current set of changes, or submit the new PR."
@@ -703,7 +691,7 @@ The value is set to fontify the values of the enum fields."
 (defvar gnats-edit-mode-hook nil
   "Hook run by `gnats-edit-mode'.")
 
-(defun gnats-edit-mode ()
+(defun gnats-edit-mode (&optional quietp)
   "Major mode for editing PRs.
 Press \\[gnats-apply-or-submit] to submit your changes."
   ;; Should we be doing this here?  By the time the mode is specified
@@ -725,7 +713,8 @@ Press \\[gnats-apply-or-submit] to submit your changes."
   (make-local-variable 'after-change-functions)
   (setq after-change-functions
 	(cons 'gnats-extend-properties after-change-functions))
-  (message "Press `C-c C-c' to submit your changes.")
+  (unless quietp
+    (message "Press `C-c C-c' to submit your changes."))
   (run-hooks 'gnats-edit-mode-hook))
 (put 'gnats-edit-mode 'mode-class 'special)
 
@@ -1312,9 +1301,11 @@ locking the database."
 	   (query-pr-buffer
 	    (generate-new-buffer " *gnats-query-pr*"))
 	   (query-pr-res
-	    (apply 'call-process
-		   "query-pr" nil query-pr-buffer nil "--print-server-addr"
-		              query-args)))
+	    (if (and server (not (string-match "^/" server)))
+		(apply 'call-process
+		       "query-pr" nil query-pr-buffer nil "--print-server-addr"
+		       query-args)
+	      (getenv "GNATSDB"))))
       (if query-pr-res
 	  (let ((buffer-contents (save-excursion
 				   (set-buffer query-pr-buffer)
@@ -1343,8 +1334,7 @@ locking the database."
 ;;;###autoload
 (defun send-pr (&optional arg)
   "Generate a new initial PR template for the user.
-If a prefix argument is given, run `gnats-change-database' first.
-"
+If a prefix argument is given, run `gnats-change-database' first."
   (interactive "P")
   (when arg
     (call-interactively 'gnats-change-database))
@@ -1414,6 +1404,7 @@ If a prefix argument is given, run `gnats-change-database' first.
   (define-key gnats-query-mode-map "D" 'gnats-change-database)
   (define-key gnats-query-mode-map "e" 'gnats-query-edit-pr)
   (define-key gnats-query-mode-map "g" 'gnats-query-reread)
+  (define-key gnats-query-mode-map "G" 'query-pr)
   (define-key gnats-query-mode-map "q" 'bury-buffer)
   (define-key gnats-query-mode-map "s" 'send-pr))
 
@@ -1448,6 +1439,7 @@ Use \\[gnats-query-view-pr] to view the problem report under the cursor and
 				 (define-key keymap "e" nil)
 				 (define-key keymap "g" nil)
 				 (define-key keymap "s" nil)
+				 (define-key keymap "\r" nil)
 				 keymap))))
   (run-hooks 'gnats-query-mode-hook))
 (put 'gnats-query-mode 'mode-class 'special)
@@ -1589,11 +1581,11 @@ The function works in the query buffers."
 QUERY is a string representing a query in the gnatsd format."
   ;; TODO: Maybe the description of the format should be here?
   (interactive (list (read-string "Query: " nil 'query-pr-history)))
-  (let ((edit-buffer (get-buffer-create gnats-query-buffer-name))
+  (let ((query-buffer (get-buffer-create gnats-query-buffer-name))
 	(success nil))
     (unwind-protect
 	(save-excursion
-	  (set-buffer edit-buffer)
+	  (set-buffer query-buffer)
 	  (gnats-clear-edit-buffer)
 	  (gnats-delete-process)
 	  (gnats-edit-server)
@@ -1608,20 +1600,34 @@ QUERY is a string representing a query in the gnatsd format."
 	  (setq success t))
       (if (not success)
 	  (progn
-	    (kill-buffer edit-buffer)
+	    (kill-buffer query-buffer)
 	    (kill-buffer (get gnats-server-conn 'server-buffer)))
-	(switch-to-buffer edit-buffer)
+	(switch-to-buffer query-buffer)
+	(message "%d matching PRs." (count-lines (point-min) (point-max)))
 	(goto-char (point-min))))))
 
+;;;###autoload
+(defun unlock-database ()
+  "Unlock the whole database."
+  (interactive)
+  (gnats-send-command "UNDB"))
+
 ;;;###autoload
 (defun unlock-pr (pr)
   "Unlock the problem report PR."
-  (interactive "nPR Number: ")
+  (interactive "sPR Number: ")
   (gnats-send-command "UNLK" pr))
 
 (defun gnats-change-database (database host port user)
   "Set new database parameters."
-  (interactive "sDatabase: \nsHost: \nnPort: \nsUser: ")
+  (interactive
+   (list (read-from-minibuffer "Database: " gnats-database)
+	 (read-from-minibuffer "Host: " (or gnats-server "localhost"))
+	 (string-to-number
+	  (read-from-minibuffer "Port: " (if gnats-port
+					     (format "%d" gnats-port)
+					   1529)))
+	 (read-from-minibuffer "User: " (or gnats-user (user-login-name)))))
   (setq gnats-database database
 	gnats-server host
 	gnats-port port
@@ -1632,6 +1638,17 @@ QUERY is a string representing a query in the gnatsd format."
 		gnats-port port
 		gnats-user user))
 
+(defun gnats-show-connection ()
+  "Show the server connection buffer associated with the current buffer.
+You can use this function to view the communication with gnatsd in case of
+problems."
+  (interactive)
+  (let ((buffer (get gnats-server-conn 'server-buffer)))
+    (if buffer
+	(switch-to-buffer buffer)
+      (error "No connection associated with this buffer"))))
+
+  
 
 
 (defvar gnats-dbconfig-mode-map nil
@@ -1660,7 +1677,7 @@ QUERY is a string representing a query in the gnatsd format."
   `(,(regexp-opt
       '("add-audit-trail" "all" "allow-any-value" "append-to-field"
 	"audit-trail-format" "aux-flags" "binary-index" "body" "builtin-name"
-	"business-day-hours" "business-week-hours" "category-dir-perms"
+	"business-day-hours" "business-week-days" "category-dir-perms"
 	"create-category-dirs" "database-info" "debug-mode" "default"
 	"description" "edit-only" "exact-regexp" "field" "fields"
 	"fixed-address" "format" "from-address" "header" "index"
@@ -1707,6 +1724,11 @@ QUERY is a string representing a query in the gnatsd format."
   (setq local-abbrev-table gnats-dbconfig-mode-abbrev-table)
   (make-local-variable 'font-lock-defaults)
   (setq font-lock-defaults '(gnats-dbconfig-font-lock-keywords nil t))
+  (setq imenu-generic-expression
+	'((nil "^\\(database-info\\|on-change\\|index\\|initial-entry\\)" 1)
+	  ("Mail formats" "^mail-format +\"\\([^\"\n]+\\)\"" 1)
+	  ("Queries" "^query +\"\\([^\"\n]+\\)\"" 1)
+	  ("Fields" "^field +\"\\([^\"\n]+\\)\"" 1)))
   (run-hooks 'gnats-dbconfig-mode-hook))
 (add-to-list 'auto-mode-alist '("\\<dbconfig$" . gnats-dbconfig-mode))
 
