@@ -1,6 +1,5 @@
 /* Remote GNATS server.
-   Copyright (C) 2000, 2001 Milan Zamazal.
-   Copyright (C) 1994, 95, 96, 1997, 1999, 2000 Free Software Foundation, Inc.
+   Copyright (C) 1994, 95, 96, 1997, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
    Contributed by Brendan Kehoe (brendan@cygnus.com).
    Further hacked by Milan Zamazal (pdm@zamazal.org).
 
@@ -254,26 +253,35 @@ match (const char *line, const char *pattern, bool matchcase)
 static int
 password_match (const char *password, const char *hash)
 {
-  if (! strncmp (hash, "$0$", 3))
+  if (strlen(password) && strlen(hash))
     {
-      /* explicit plain-text password */
-      return ! strcmp (password, hash+3);
-    }
-  else if (! strncmp (hash, "$1$", 3))
-    {
-      /* MD5 hash of the password */
+      if (! strncmp (hash, "$0$", 3))
+	{
+	  /* explicit plain-text password */
+	  return match (password, hash+3, TRUE);
+	}
+      else
+	{
 #ifdef HAVE_LIBCRYPT
-      char *encrypted = crypt (password, hash);
-      return encrypted && ! strcmp (encrypted, hash);
+	  /* DES crypt or MD5 hash of the password */
+	  char *encrypted = crypt (password, hash);
+	  return encrypted && ! strcmp (encrypted, hash);
 #else
-      /* TODO: log some warning */
-      return FALSE;
+	  /* TODO: log some warning */
+	  return FALSE;
 #endif
+	}
     }
   else
     {
-      /* default password type is plain-text */
-      return match (password, hash, TRUE);
+      if (strlen(password))
+	{
+	  return FALSE;
+	}
+      else
+	{
+	  return ! strlen(hash) ;
+	}
     }
 }
 
@@ -288,7 +296,7 @@ get_name (struct in_addr *host)
   char **pp;
 #endif
 
-  hp = gethostbyaddr ((char *) host, sizeof (host), AF_INET);
+  hp = gethostbyaddr ((char *) host, sizeof (*host), AF_INET);
   if (hp == NULL)
     {
       return NULL;
@@ -456,8 +464,11 @@ findUserAccessLevel (const char *file, const char *user, const char *passwd,
 	      if (! password_match (passwd, ent->admFields[1]))
 		{
 		  /* Username matched but password didn't.  */
-		  *access = ACCESS_NONE;
-		  found = 1;
+		  if (strlen(ent->admFields[1]) && strlen(passwd))
+		    {
+		      *access = ACCESS_NONE;
+		      found = 1;
+		    }
 		}
 	      else
 		{
@@ -466,7 +477,10 @@ findUserAccessLevel (const char *file, const char *user, const char *passwd,
 		      /* Compare all given names against the name of the
 			 requested database. */
 		      const char *l2 = ent->admFields[3];
-
+		      
+		      if (! strlen(l2))
+			found = 1;
+		      
 		      while (l2 != NULL && ! found)
 			{
 			  char *token = get_next_field (&l2, ',');
