@@ -267,7 +267,7 @@ getFieldHeader (char **linePtr, size_t *headerLen, size_t linelen)
   char *res = *linePtr;
 
   /* Grab the first word ending in space or : */
-  while (line < lineEnd && (! isspace ((int) *line)) && *line != ':')
+  while (line < lineEnd && (! isspace ((int)(unsigned char) *line)) && *line != ':')
     {
       line++;
     }
@@ -280,7 +280,7 @@ getFieldHeader (char **linePtr, size_t *headerLen, size_t linelen)
 
   *headerLen = line - res;
 
-  while (line < lineEnd && isspace ((int) *line))
+  while (line < lineEnd && isspace ((int)(unsigned char) *line))
     {
       line++;
     }
@@ -383,7 +383,7 @@ addLineToPR (PR *pr, char *buffer, char *line, size_t linelen, int prune)
 
 	  /* Skip leading spaces; I don't think this is really needed
 	     anyway, since we've already done it in getFieldHeader ().  */
-	  while (linelen > 0 && isspace ((int) *line))
+	  while (linelen > 0 && isspace ((int)(unsigned char) *line))
 	    {
 	      line++;
 	      linelen--;
@@ -391,7 +391,7 @@ addLineToPR (PR *pr, char *buffer, char *line, size_t linelen, int prune)
 
 	  /* Fields that aren't MultiText shouldn't have spaces
 	     at the end.  */
-	  while (linelen > 0 && isspace ((int) line[linelen - 1]))
+	  while (linelen > 0 && isspace ((int)(unsigned char) line[linelen - 1]))
 	    {
 	      linelen--;
 	    }
